from qutrunk.core.circuit import QCircuit
from qutrunk.core.printer import Printer
from qutrunk.core.gates import (
    H, Measure, CNOT, Toffoli, P, R, Rx, Ry, Rz, S, Sdg, T, Tdg, 
    X, Y, Z, NOT, Swap, SqrtSwap, SqrtX, All, C, Rxx, Ryy, Rzz, U1, U2, U3,
    Barrier
)
from numpy import pi
from qutrunk.core.exporter import Exporter

def run_gates():
    # allocate
    qc = QCircuit()
    qr = qc.allocate(3)

    # apply gate
    H | qr[0]
    CNOT | (qr[0], qr[1])
    NOT | qr[0]
    Toffoli | (qr[0], qr[1], qr[2])
    P(pi/2) | qr[2]
    R(pi/2) | qr[0]
    Rx(pi/2) | qr[1]
    Ry(pi/2) | qr[1]
    Rz(pi/2) | qr[1]
    S | qr[0]
    Sdg | qr[0]
    T | qr[0]
    Tdg | qr[0]
    X | qr[2]
    Y | qr[2]
    Z | qr[2]
    Swap | (qr[0], qr[1])
    SqrtSwap | (qr[0], qr[1])
    SqrtX | qr[0]
    C(Y) | (qr[0], qr[1])
    C(Z) | (qr[0], qr[1])
    C(P(pi/2)) | (qr[0], qr[1])
    C(Rx(pi/2)) | (qr[0], qr[1])
    C(Ry(pi/2)) | (qr[0], qr[1])
    C(Rz(pi/2)) | (qr[0], qr[1])
    C(X, 2) | (qr[0], qr[1], qr[2])
    Rxx(pi/2) | (qr[0], qr[1])
    Ryy(pi/2) | (qr[0], qr[1])
    Rzz(pi/2) | (qr[0], qr[1])

    U1(pi/2) | qr[0]
    U2(pi/2, pi/2) | qr[0]
    U3(pi/2, pi/2, pi/2) | qr[0]

    # 验证抛异常
    # C(H) | (qr[0], qr[1])

    Barrier | qr

    All(Measure) | qr

    # print circuit
    Printer.print_circuit(qc)

    qc.run()

    # print measure result
    # print([int(q) for q in qr])


if __name__ == '__main__':
    run_gates()