import math
import random

from numpy import pi

from qutrunk.core.circuit import QCircuit, InitState
from qutrunk.core.gates import H, X, C, Z
from qutrunk.core.counter import Counter
from qutrunk.core.backends import BackendQuSim
from qutrunk.core.exporter import Exporter


def apply_oracle(qr, num_qubits, sol_elem):
    for q in range(num_qubits):
        if ((sol_elem >> q) & 1) == 0:
             X | qr[q]

    ctrls = []
    for q in range(num_qubits):
        ctrls.append(qr[q])
    C(Z, len(ctrls) - 1) | (tuple(ctrls))

    for q in range(num_qubits):
        if ((sol_elem >> q) & 1) == 0:
             X | qr[q]


def apply_diffuser(qr, num_qubits):
    for q in range(num_qubits):
        H | qr[q]

    for q in range(num_qubits):
        X | qr[q] 
    
    ctrls = []
    for q in range(num_qubits):
        ctrls.append(qr[q])
    C(Z, len(ctrls) - 1) | (tuple(ctrls))

    for q in range(num_qubits):
        X | qr[q]  

    for q in range(num_qubits):
        H | qr[q]


def run_grover(qubits=15, backend=None):
    num_qubits = qubits
    num_elems = 2 ** num_qubits
    num_reps = math.ceil(pi / 4 * math.sqrt(num_elems))
    print("num_qubits:", num_qubits, "num_elems:", num_elems, "num_reps:", num_reps)

    sol_elem = random.randint(0, num_elems)

    circuit = QCircuit(backend=backend)
    qureg = circuit.allocate(num_qubits, InitState.Plus)
    # 将统计信息归集到Counter类
    ct = Counter(circuit)

    for _ in range(num_reps):
        apply_oracle(qureg, num_qubits, sol_elem)
        apply_diffuser(qureg, num_qubits)
        prob_amp = circuit.get_prob_amp(sol_elem)
        print(f"prob of solution |{sol_elem}> = {prob_amp}")

    # 运行线路
    circuit.run()
    ct.show_verbose()

    return circuit


if __name__ == '__main__':
    run_grover()

