from qutrunk.core.circuit import QCircuit, InitState
from qutrunk.core.printer import Printer
from qutrunk.core.gates import CNOT, Measure, C, NOT, Toffoli, X, All


# 自增运行
def increment(num_qubits, initvalue):
    circuit = QCircuit()
    qr = circuit.allocate(num_qubits, InitState.Classical, initvalue)

    for i in range(num_qubits-1):
        ctrl_bits = qr[:num_qubits-1-i]
        C(NOT, num_qubits-1-i) | (*ctrl_bits, qr[num_qubits-1-i])
    X | qr[0]

    # 等同于下面的代码
    # C(NOT, 3) | (qr[0], qr[1], qr[2], qr[3])
    # Toffoli | (qr[0], qr[1], qr[2])
    # CNOT | (qr[0], qr[1])
    # X | qr[0]

    All(Measure) | qr
    res = circuit.run()
    print(res.get_outcome())

# 自减运行
def decrement(num_qubits, initvalue):
    circuit = QCircuit()
    qr = circuit.allocate(num_qubits, InitState.Classical, initvalue)

    X | qr[0]
    for i in range(num_qubits-1):
        ctrl_bits = qr[:1+i]
        C(NOT, 1+i) | (*ctrl_bits, qr[1+i])

    # 等同于下面的代码
    # X | qr[0]
    # CNOT | (qr[0], qr[1])
    # Toffoli | (qr[0], qr[1], qr[2])
    # C(NOT, 3) | (qr[0], qr[1], qr[2], qr[3])

    All(Measure) | qr
    res = circuit.run()
    print(res.get_outcome())

if __name__ == '__main__':
    increment(4, 0)
    # decrement(4, 0b0001)
