from qutrunk.core.circuit import QCircuit, InitState
from qutrunk.core.gates import H, X, C, Z, All, Measure, CNOT, U3
from qutrunk.core.counter import Counter
from qutrunk.core.printer import Printer


def run_teleport():
    # allocate
    qc = QCircuit()
    qureg = qc.allocate(3)

    # Prepare an initial state
    #
    U3(0.3, 0.2, 0.1) | qureg[0]

    # Prepare a Bell pair
    H | qureg[1]
    CNOT | (qureg[1], qureg[2])

    # Measure in the Bell basis
    CNOT | (qureg[0], qureg[1])
    H | qureg[0]
    Measure | qureg[0]
    Measure | qureg[1]

    # Apply a correction
    Z | qureg[2]
    X | qureg[2]
    Measure | qureg[2]

    # print circuit
    # Printer.print_circuit(qc)

    # run circuit
    res = qc.run(shots=1024)

    print(res.get_counts())
    # return qureg


if __name__ == '__main__':
    run_teleport()