from qutrunk.parser.qasmparser import QasmParser
from qutrunk.core.circuit import QCircuit


class Qasm:
    """OPENQASM circuit object."""
    def __init__(self, filename: str = ""):
        if filename is None:
            raise FileNotFoundError("File not found.")
        self.filename = filename
        self.cmds = []

    def parse(self):
        """parse the data"""
        with open(file=self.filename, encoding="utf-8") as file:
            for _ in range(5):
                next(file)
            for line in file:
                if line == "\n":
                    continue
                # 调用QasmParse解析指令,并保存解析的结果
                self.cmds.append(QasmParser.parse(line))

    def get_qubits(self):
        """Get the number of qubits"""
        with open(file=self.filename, encoding="utf-8") as file:
            for line in file:
                if "qreg" in line:
                    qubits = list(map(int, list(filter(str.isdigit, line))))
                    return qubits[0]

    def run(self):
        """send the quantum circuit to qubox through rpc"""
        circuit = QCircuit()
        qr = circuit.allocate(self.get_qubits())
        circuit.set_cmds(self.cmds)
        circuit.flush()

        # 获取测量结果
        print([int(q) for q in qr])

    # TODO:获取量子算法执行的结果


if __name__ == "__main__":
    qm = Qasm(filename="qasm.txt")
    qm.parse()
    qm.run()

