import math

from qutrunk.core.command import Command


def h_gate(cmd):
    # h q[0];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="H", targets=targets, qasm=cmd)


def measure_gate(cmd):
    # measure q[0] -> c[0];
    targets = list(map(int, list(filter(str.isdigit, cmd.split(" ")[1]))))
    return Command(gate="Measure", targets=targets, qasm=cmd)


def cx_gate(cmd):
    # cx q[1],q[2];
    c = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="CX", controls=c[:1], targets=c[1:], qasm=cmd)


def toffoli_gate(cmd):
    # ccx q[0],q[1],q[2];
    c = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="CX", controls=c[:2], targets=c[2:], qasm=cmd)


def p_gate(cmd):
    # p(pi/2) q[1];
    targets = list(map(int, list(filter(str.isdigit, cmd.split(" ")[1]))))
    return Command(gate="P", rotation=[math.pi/2], targets=targets, qasm=cmd)


def rx_gate(cmd):
    # rx(pi/2) q[1];
    targets = list(map(int, list(filter(str.isdigit, cmd.split(" ")[1]))))
    return Command(gate="Rx", rotation=[math.pi/2], targets=targets, qasm=cmd)


def ry_gate(cmd):
    # ry(pi/2) q[2];
    targets = list(map(int, list(filter(str.isdigit, cmd.split(" ")[1]))))
    return Command(gate="Ry", rotation=[math.pi/2], targets=targets, qasm=cmd)


def rz_gate(cmd):
    # rz(pi/2) q[0];
    targets = list(map(int, list(filter(str.isdigit, cmd.split(" ")[1]))))
    return Command(gate="Rz", rotation=[math.pi/2], targets=targets, qasm=cmd)


def s_gate(cmd):
    # s q[0];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="S", targets=targets, qasm=cmd)


def sdg_gate(cmd):
    # sdg q[1];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="Sdg", targets=targets, qasm=cmd)


def t_gate(cmd):
    # t q[2];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="T", targets=targets, qasm=cmd)


def tdg_gate(cmd):
    # tdg q[2];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="Tdg", targets=targets, qasm=cmd)


def x_gate(cmd):
    # x q[0];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="X", targets=targets, qasm=cmd)


def y_gate(cmd):
    # y q[2]
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="Y", targets=targets, qasm=cmd)


def z_gate(cmd):
    # z q[1];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="Z", targets=targets, qasm=cmd)


def swap_gate(cmd):
    # swap q[1],q[2];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="Swap", targets=targets, qasm=cmd)


def sx_gate(cmd):
    # sx q[1];
    targets = list(map(int, list(filter(str.isdigit, cmd))))
    return Command(gate="SqrtX", targets=targets, qasm=cmd)


def rxx_gate(cmd):
    # rxx(pi/2) q[1],q[2];
    targets = list(map(int, list(filter(str.isdigit, cmd.split(" ")[1]))))
    return Command(gate="Rxx", rotation=[math.pi/2], targets=targets, qasm=cmd)


def rzz_gate(cmd):
    # rzz(pi/2) q[1],q[2];
    targets = list(map(int, list(filter(str.isdigit, cmd.split(" ")[1]))))
    return Command(gate="Rzz", rotation=[math.pi/2], targets=targets, qasm=cmd)


# def u3_gate(cmd):
#     # u(pi/2,pi/2,pi/2) q[2];
#     targets = list(map(int, list(filter(str.isdigit, cmd.split(" ")[1]))))
#     return Command(gate="U3", rotation=[math.pi/2, math.pi/2, math.pi/2], targets=targets, qasm=cmd)


# SqrtSwap | (qr[0], qr[1])
# TODO:后期添加
# C(Y) | (qr[0], qr[1])
# C(Z) | (qr[0], qr[1])
# C(P(pi/2)) | (qr[0], qr[1])
# C(Rx(pi/2)) | (qr[0], qr[1])
# C(Ry(pi/2)) | (qr[0], qr[1])
# C(Rz(pi/2)) | (qr[0], qr[1])
# C(X, 2) | (qr[0], qr[1], qr[2])
# Ryy(pi/2) | (qr[0], qr[1])

# U2(pi/2, pi/2) | qr[0]
# U1(pi/2) | qr[0]
# 定义指令对应的解释方法
cmd_map = {
    "h": h_gate,
    "measure": measure_gate,
    "cx": cx_gate,
    "ccx": toffoli_gate,
    "p": p_gate,
    "rx": rx_gate,
    "ry": ry_gate,
    "rz": rz_gate,
    "s": s_gate,
    "sdg": sdg_gate,
    "t": t_gate,
    "tdg": tdg_gate,
    "x": x_gate,
    "y": y_gate,
    "z": z_gate,
    "swap": swap_gate,
    "sx": sx_gate,
    "rxx": rxx_gate,
    "rzz": rzz_gate,
    # "u": u3_gate,
}


class QasmParser:
    """OPENQASM Parser"""
    @classmethod
    def parse(cls, data):
        d = data.split(" ")
        if "(" not in d[0]:
            c = cmd_map.get(d[0], None)
            if c is None:
                raise ValueError(f"{d[0]} has not been realized.")
            return c(data.strip())
        if "(" in d[0]:
            c = cmd_map.get(d[0].split("(")[0], None)
            if c is None:
                raise ValueError(f"{d[0]} has not been realized.")
            return c(data.strip())
