import platform

from qutrunk.tools.function_time import timefn
try:
    from qutrunk.sim.local import simulator
except ImportError:  # pragma: no cover
    # windows 
    from qutrunk.sim.local.Release import simulator


class BackendLocalService:
    """ Simulator is a compiler engine which simulates a quantum computer using C++-based kernels."""
    @timefn
    def init(self, qubits, state, value, show):
        return simulator.init(qubits, state, value, show)

    @timefn
    def send_circuit(self, circuit, final):
        start = circuit.cmd_cursor
        cmds = circuit.cmds[start:]
        temp_cmds = []
        for cmd in cmds:
            tempCmd = simulator.Cmd()
            tempCmd.gate = str(cmd.gate)
            tempCmd.targets = cmd.targets
            tempCmd.controls = cmd.controls
            tempCmd.rotation = cmd.rotation
            tempCmd.desc = cmd.qasm 
            temp_cmds.append(tempCmd)
        simulator.send_circuit(temp_cmds, final)
        # start = circuit.cmd_cursor
        # stop = len(circuit.cmds)
        # for idx in range(start, stop):
        #     cmd = circuit.cmds[idx]
        #     res = simulator.execCmd(gate=str(cmd.gate),
        #                             targets=cmd.targets,
        #                             controls=cmd.controls,
        #                             rotation=cmd.rotation,
        #                             desc=cmd.qasm)

        #     if str(cmd.gate) == "Measure":
        #         qubit = cmd.targets[0]
        #         circuit.qreg[qubit].cbit = res

    @timefn
    def run(self, shots):
        return simulator.run(shots)

    @timefn
    def get_prob_amp(self, qubit):
        return simulator.getProbOfAmp(qubit)

    @timefn
    def get_prob_outcome(self, qubit, outcome):
        return simulator.getProbOfOutcome(qubit, outcome)

    @timefn
    def get_prob_all_outcome(self, qubit):
        return simulator.getProbOfAllOutcome(qubit)

    @timefn
    def get_all_state(self):
        return simulator.getAllState()

    @timefn
    def apply_QFT(self, qubits):
        return simulator.apply_QFT(qubits)

    @timefn
    def apply_Full_QFT(self):
        return simulator.apply_Full_QFT()