import uuid
from thrift.transport import TSocket
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TMultiplexedProtocol

from qutrunk.sim.qusim.qusim import QuSimServer
from qutrunk.sim.qusim.qusim import ttypes as qusim
from qutrunk.tools.function_time import timefn


class QuSimApiServer:
    def __init__(self, ip='localhost', port=9090):
        socket = TSocket.TSocket(ip, port)
        self._transport = TTransport.TBufferedTransport(socket)
        protocol = TBinaryProtocol.TBinaryProtocol(self._transport)
        quest = TMultiplexedProtocol.TMultiplexedProtocol(protocol, "QuSimServer")
        self._client = QuSimServer.Client(quest)
        self._taskid = uuid.uuid4().hex
        self._transport.open()

    def close(self):
        self._transport.close()

    @timefn
    def init(self, qubits, state, value, density):
        req = qusim.InitQubitsReq(self._taskid, qubits, state, value, density, 0)
        return self._client.initQubits(req)

    @timefn
    def send_circuit(self, circuit, final):
        req = qusim.SendCircuitCmdReq(self._taskid, circuit, final)
        res = self._client.sendCircuitCmd(req)
        return res

    @timefn
    def run(self, shots):
        req = qusim.RunCircuitReq(self._taskid, shots)
        res = self._client.run(req)
        return res.result

    @timefn
    def get_prob_amp(self, index):
        req = qusim.GetProbAmpReq(self._taskid, index)
        res = self._client.getProbAmp(req)
        return res.amp

    @timefn
    def get_prob_outcome(self, qubit, outcome):
        req = qusim.GetProbOfOutcomeReq(self._taskid, qubit, outcome)
        res = self._client.getProbOfOutcome(req)
        return res.pro_outcome

    @timefn
    def get_prob_all_outcome(self, qubits):
        req = qusim.GetProbOfAllOutcomReq(self._taskid, qubits)
        res = self._client.getProbOfAllOutcome(req)
        return res.pro_outcomes

    @timefn
    def get_all_state(self):
        req = qusim.GetAllStateReq(self._taskid)
        res = self._client.getAllState(req)
        return res.all_state

    @timefn
    def cancel_Cmd(self):
        req = qusim.CancelCmdReq(self._taskid)
        return self._client.cancelCmd(req)

    @timefn
    def apply_QFT(self, qubits):
        req = qusim.ApplyQFTReq(self._taskid, qubits)
        res = self._client.applyQFT(req)

    @timefn
    def apply_Full_QFT(self):
        req = qusim.ApplyFullQFTReq(self._taskid)
        res = self._client.applyFullQFT(req)
