# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qutrunk',
 'qutrunk.core',
 'qutrunk.core.backends',
 'qutrunk.core.backends._ibm',
 'qutrunk.example',
 'qutrunk.parser',
 'qutrunk.qasm',
 'qutrunk.qasm.node',
 'qutrunk.sim',
 'qutrunk.sim.local',
 'qutrunk.sim.local.pybind11',
 'qutrunk.sim.local.pybind11.docs',
 'qutrunk.sim.local.pybind11.pybind11',
 'qutrunk.sim.local.pybind11.tests',
 'qutrunk.sim.local.pybind11.tests.extra_python_package',
 'qutrunk.sim.local.pybind11.tests.extra_setuptools',
 'qutrunk.sim.local.pybind11.tests.test_cmake_build',
 'qutrunk.sim.local.pybind11.tests.test_embed',
 'qutrunk.sim.local.pybind11.tools',
 'qutrunk.sim.qucloud',
 'qutrunk.sim.qusim',
 'qutrunk.sim.qusim.code',
 'qutrunk.sim.qusim.qusim',
 'qutrunk.test',
 'qutrunk.tools']

package_data = \
{'': ['*'],
 'qutrunk': ['config/*'],
 'qutrunk.qasm': ['libs/*'],
 'qutrunk.sim.local': ['QuEST/*',
                       'QuEST/include/*',
                       'QuEST/src/*',
                       'QuEST/src/CPU/*',
                       'QuEST/src/GPU/*',
                       'Release/*',
                       'Release/Release/*'],
 'qutrunk.sim.local.pybind11': ['include/pybind11/*',
                                'include/pybind11/detail/*',
                                'include/pybind11/stl/*'],
 'qutrunk.sim.local.pybind11.docs': ['_static/*',
                                     'advanced/*',
                                     'advanced/cast/*',
                                     'advanced/pycpp/*',
                                     'cmake/*'],
 'qutrunk.sim.local.pybind11.tests.test_cmake_build': ['installed_embed/*',
                                                       'installed_function/*',
                                                       'installed_target/*',
                                                       'subdirectory_embed/*',
                                                       'subdirectory_function/*',
                                                       'subdirectory_target/*']}

install_requires = \
['numpy>=1.22.3,<2.0.0',
 'ply>=3.11,<4.0',
 'pyyaml>=6.0,<7.0',
 'requests>=2.27.1,<3.0.0',
 'thrift>=0.15.0,<0.16.0']

setup_kwargs = {
    'name': 'qutrunk',
    'version': '0.1.8',
    'description': 'qutrunk is an open source library for quantum computing',
    'long_description': '## qutrunk\nqutrunk is an quantum programming framework developed by QuDoor corporation, provide the required APIs for quantum programming, it works as follow: \n\nExport quantum programs based on quantum circuit, qubit and quantum gate, etc. Then you can run the \nquantum circuit by the following two ways:\n1. Use PC for quantum computing(work as the default way). \n2. Connect with QuBox(a quantum device developed by qudoor), to run quantum programs, it provide more powerful quantum computing. \n\n## Install\n1. Install from whl package, run the following command directly:\n```\npip install qutrunk\n```\n\n2. Install from source code, run the following command to install qutrunk by source code:\n```\npython3 setup.py install\n```\n\n## Example:\nbell-pair quantum algorithm：\n\n```\nfrom qutrunk.core.circuit import QCircuit\nfrom qutrunk.core.printer import Printer\nfrom qutrunk.core.gates import H, CNOT, Measure\n\nqc = QCircuit()\nqr = qc.allocate(2) # allocate\n\nH | qr[0]   # apply gate\nCNOT | (qr[0], qr[1])\nMeasure | qr[0]\nMeasure | qr[1]\n\nPrinter.print_circuit(qc)   # print circuit\nres = qc.run(shots=1024) # run circuit\nprint(res.get_counts()) # print result\n```\n\n ## License\n Apache Licence 2.0\n\n Copyright (c) 2022-2025 By QUDOOR Technolgies Inc. All Right Reserved.',
    'author': 'qudoorzh2022',
    'author_email': 'qudoorzh2022@163.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://www.qudoor.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}
from build import *
build(setup_kwargs)

setup(**setup_kwargs)
